
/*
 *  cmp.c
 *  Copyright (C) 1999 Brad Proctor
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: cmp.c,v 1.3 1999/06/21 15:04:55 brad Exp $
 */

#include <stdio.h>
#include <string.h>
#include "version.h"

#define BUF_SIZE  32768

static char help_text[] =
{
   "Usage: cmp [-l|-s] file1 file2\n\n"
   "  -l         \n"
   "  -s         Suppress output, exit status only\n"
   "  --help     Print this help message\n"
   "  --version  Print the program version\n\n"
   "Report bugs to fredlie@sprynet.com"
};

static char buf1[BUF_SIZE];
static char buf2[BUF_SIZE];
static char flag_quiet = 0;
static char flag_l = 0;
static unsigned long bytes = 1;
static unsigned long lines = 1;
static FILE *file1 = 0;
static FILE *file2 = 0;
static char *name1 = 0;
static char *name2 = 0;

/*
 * Main
 */

int main(int argc, char **argv)
{
   int i;
   char *ptr;
   
   for(i = 1; i < argc; ++i)
   {
      ptr = argv[i];
      if(*ptr == '-')
      {
         while(*++ptr)
         switch(*ptr)
         {
            case '-':
               ++ptr;
               if(!strcmp(argv[1], "--help"))
               {
                  puts(help_text);
                  return (0);
               }
               else if(!strcmp(argv[1], "--version"))
               {
                  puts("cmp (butils) "VERSION_STRING);
                  return (0);
               }
               else if(*ptr == '\0')
               {
                  if(!file1)
                     file1 = stdin;
                  else if(!file2)
                     file2 = stdin;
               }
               break;
            case 'l': flag_l = 1; break;
            case 's': flag_quiet = 1; break;
            default:
         }
      }
      else if(!file1)
      {
         if((file1 = fopen(argv[i], "r")) == NULL)
         {
            fprintf(stderr, "%s: Could not open `%s' for reading.\n",
               argv[0], argv[i]);
            fputs("Try `cmp --help' for more information.\n", stderr);
            return (2);
         }
         name1 = argv[i];
      }
      else if(!file2)
      {
         if((file2 = fopen(argv[i], "r")) == NULL)
         {
            fprintf(stderr, "%s: Could not open `%s' for reading.\n",
               argv[0], argv[i]);
            fputs("Try `cmp --help' for more information.\n", stderr);
            return (2);
         }
         name2 = argv[i];
      }
   }
   if(!file1 && !file2)
   {
      fprintf(stderr, "%s: Missing operand.\n", argv[0]);
      fputs("Try `cmp --help' for more information.\n", stderr);
      return (2);
   }
   if(flag_quiet)
   {
      while(1)
      {
         fread(buf1, 1, BUF_SIZE, file1);
         fread(buf2, 1, BUF_SIZE, file2);          
         if(memcmp(buf1, buf2, BUF_SIZE))
            return (1);
      }
   }
   else if(flag_l)
   {
      int b, c;
      while(1)
      {
         b = getc(file1);
         c = getc(file2);
         if(c == EOF)
         {
            printf("%s: EOF on %s\n", argv[0], name1);
            break;
         }
         else if(b == EOF)
         {
            printf("%s: EOF on %s\n", argv[0], name2);
            break;
         }
         else if(c != b)
            printf("%7ld %3o %3o\n", bytes, b, c);
         ++bytes;
      }
   }
   else
   {
      int b, c;
      while(1)
      {
         b = getc(file1);
         c = getc(file2);
         if(c != b)
            break;
         if(c == '\n')
            ++lines;
         ++bytes;
      }
      printf("%s %s differ: char %ld, line %ld\n", 
         name1, name2, bytes, lines);
   }
   return (0);
}